#import "AppDelegate.h"

#import <PushTechChatSDK/PushTechChatSDK.h>

#import <MediaPlayer/MediaPlayer.h>
#import <AddressBookUI/AddressBookUI.h>

#import <MWPhotoBrowser.h>
#import <CMNavBarNotificationView.h>
#import <UIImageView+WebCache.h>

#import <MBProgressHUD.h>

#import "PSHCampaignWebViewController.h"

@interface UIButton (PSHDemoApp)

@end

@implementation UIButton (PSHDemoApp)

-(void)setTitleFont:(UIFont*)titleFont
{
    self.titleLabel.font = titleFont;
}

@end


@interface UITableViewCell (PSHDemoApp)

@end

@implementation UITableViewCell (PSHDemoApp)

-(void)setTextFont:(UIFont*)titleFont
{
    self.textLabel.font = titleFont;
}

-(void)setDetailFont:(UIFont*)titleFont
{
    self.detailTextLabel.font = titleFont;
}

@end


@interface AppDelegate () <UIApplicationDelegate, PSHChatViewDelegate, MWPhotoBrowserDelegate>

@property (nonatomic, strong) MWPhoto *currentPhoto;
@property (nonatomic, strong) UITabBarController *chatTabBarController;
@property (nonatomic, strong) PSHChatViewController *chatViewController;

@property (nonatomic, assign) BOOL appIsInForeground;
@property (nonatomic, strong) PSHCampaignDAO *pendingCampaign;
@property (nonatomic, assign) BOOL isShowingCampaign;

@property (nonatomic, strong) MBProgressHUD *hud;

@end


@implementation AppDelegate


-(BOOL)application:(UIApplication*)application didFinishLaunchingWithOptions:(NSDictionary*)launchOptions
{
    [application setStatusBarHidden:NO];
    
    _appIsInForeground = application.applicationState == UIApplicationStateActive;
    _isShowingCampaign = NO;
    
    // initialize engine

    [PSHChatEngine initializeWithAppId:@"YOUR-APP-ID"
                             appSecret:@"YOUR-APP-SECRET"
                              logLevel:PSHLogLevelDebug];
    
    [[PSHBusProvider sharedInstance] addListener:self];
    
    [self registerForPushNotifications];
    
    [self setupAppearance];
    
    [self setupControllers];
    
    
    NSDictionary *pushNotification = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if (pushNotification) {
        [self handleRemotePushNotification:pushNotification completion:NULL];
    }
    
    return YES;
}

- (void)setupControllers
{
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    self.window.backgroundColor = [UIColor whiteColor];
    
    PSHChatEngine* engine = [PSHChatEngine sharedInstance];
    
    // present example chat view
    /*
     _chatViewController = [engine instantiateChatViewController];
     _chatViewController.delegate = self;
     _chatViewController.chatId = @"34622174612@pushtech.com";
     self.window.rootViewController = _chatViewController;
     */
    
    // present main complete GUI
    _chatTabBarController = [engine instantiateMainTabBarViewController];
    //_chatTabBarController.selectedIndex = 1; // appear showing the chat list
    self.window.rootViewController = _chatTabBarController;
    
    [self.window makeKeyAndVisible];
    
    if ([PSHEngine sharedInstance].isAppRegistered) {
        if (engine.isChatEnabled) {
            if (!engine.isInDomain) {
                [self presentRegistrationUI];
            }
            else {
                [self updateBadges];
            }
        }
    }
    else {
        // block UI until we get the app universe
        _hud = [MBProgressHUD showHUDAddedTo:_chatTabBarController.view animated:YES];
        _hud.dimBackground = YES;
    }
}

- (void)setupAppearance
{
    UIColor* mainBlack = [UIColor colorWithRed:51/255.0 green:51/255.0 blue:51/255.0 alpha:1.0];
    UIColor* mainBlue = [UIColor colorWithRed:43/255.0 green:203/255.0 blue:240/255.0 alpha:1.0];
    UIColor* mainGrey = [UIColor colorWithRed:240/255.0 green:240/255.0 blue:240/255.0 alpha:1.0];
    
    if ([[[UIDevice currentDevice] systemVersion] floatValue] >= 7) {
        [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleLightContent];
        
        [[UINavigationBar appearance] setBarTintColor:mainBlack];
        [[UINavigationBar appearance] setTintColor:[UIColor whiteColor]];
        
        [[UIToolbar appearance] setBarTintColor:[UIColor lightGrayColor]];
        [[UIToolbar appearance] setTintColor:[UIColor whiteColor]];
        
        [[UITabBar appearance] setBarTintColor:[UIColor lightGrayColor]];
        [[UITabBar appearance] setTintColor:[UIColor whiteColor]];
    }
    else {
        [[UINavigationBar appearance] setTintColor:mainBlack];
        
        [[UIToolbar appearance] setTintColor:[UIColor lightGrayColor]];
        
        [[UITabBar appearance] setTintColor:[UIColor lightGrayColor]];
        [[UITabBar appearance] setSelectedImageTintColor:[UIColor whiteColor]];
    }
    
    NSDictionary* navigationBarTitleStyle = @{
                                              NSForegroundColorAttributeName:[UIColor whiteColor],
                                              NSFontAttributeName:[UIFont fontWithName:@"TradeGothic-Light" size:22.0f]
                                              };
    NSDictionary* navigationBarButtons = @{
                                           NSForegroundColorAttributeName:mainBlue,
                                           NSFontAttributeName:[UIFont fontWithName:@"TradeGothic-Light" size:18.0f]
                                           };
    
    [[UINavigationBar appearance] setTitleTextAttributes:navigationBarTitleStyle];
    
    [[UIBarButtonItem appearanceWhenContainedIn:[UINavigationBar class], nil]
     setTitleTextAttributes:navigationBarButtons forState:UIControlStateNormal];
    
    [[UIBarButtonItem appearanceWhenContainedIn:[UINavigationBar class], nil]
     setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor lightGrayColor]} forState:UIControlStateDisabled];
    
    [[UIButton appearance] setTitleColor:mainBlue forState:UIControlStateNormal];
    [[UIButton appearance] setTitleColor:[UIColor lightGrayColor] forState:UIControlStateDisabled];
    [[UIButton appearanceWhenContainedIn:[UISearchBar class], nil] setTitleColor:[UIColor darkGrayColor] forState:UIControlStateNormal];
    
    [[UITabBar appearance] setTintColor:mainBlue];
    [[UITabBar appearance] setSelectedImageTintColor:mainBlue];
    [[UITabBar appearance] setBarTintColor:mainGrey];
    [[UITabBar appearance] setBarStyle:UIBarStyleBlackOpaque];
    
    [[UITabBarItem appearance]
     setTitleTextAttributes:@{NSForegroundColorAttributeName: mainBlack,
                              NSFontAttributeName:[UIFont fontWithName:@"TradeGothic-Light" size:12.0f]
                              }
     forState:UIControlStateNormal];
    [[UITabBarItem appearance] setTitleTextAttributes:@{NSForegroundColorAttributeName: mainBlue} forState:UIControlStateSelected];
}

- (void)presentRegistrationUI
{
    UINavigationController* registrationController = [[PSHChatEngine sharedInstance] instantiateRegistrationNavigationController];
    [self.window.rootViewController presentViewController:registrationController animated:YES completion:nil];
}

#pragma mark - App state

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    _appIsInForeground = YES;
    
    if (_pendingCampaign) {
        [self showCampaign:_pendingCampaign];
        _pendingCampaign = nil;
    }
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    _appIsInForeground = NO;
}

#pragma mark - Push notifications

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo
{
    [self handleRemotePushNotification:userInfo completion:NULL];
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult))completionHandler
{
    [self handleRemotePushNotification:userInfo completion:^{
        completionHandler(UIBackgroundFetchResultNewData);
    }];
}

- (void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken
{
    NSLog(@"did register APNS token: %@", deviceToken);
    [[PSHEngine sharedInstance] registerPushToken:deviceToken];
}

- (void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error
{
    //
}

#pragma mark - PSHBusProvider event listeners

- (void)onSuccessfulAppRegistrationBusEvent:(NSNotification *)notification
{
    NSLog(@"App is registered, setting up main UI.");
    if (_hud) {
        [MBProgressHUD hideHUDForView:_chatTabBarController.view animated:YES];
        _hud = nil;
    }
    
    if ([PSHChatEngine sharedInstance].isChatEnabled) {
        NSLog(@"Chat is enabled, will present registration UI.");
        
        [self presentRegistrationUI];
    }
    else {
        NSLog(@"Chat is disabled.");
    }
}

- (void)onShouldRefreshChatListBusEvent:(NSNotification *)notification
{
    [self updateBadges];
    
    if ([PSHChatEngine sharedInstance].isChatListVisible) {
        return;
    }
    
    if ([PSHChatEngine sharedInstance].numberOfChatsWithUnreadMessages == 0) {
        return;
    }
    
    PSHShouldRefreshChatListBusEvent* event = notification.object;
    if (event.firedByRead) {
        return;
    }
    
    PSHChatDAO *mostRecentChat = [PSHChatEngine sharedInstance].mostRecentChat;
    
    // make an status bar notification ala whatsapp
    static UIImage* notificationPlaceholderImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        notificationPlaceholderImage = [UIImage imageNamed:@"default_avatar.png"];
    });
    UIImage *notificationViewThumbnail =
        mostRecentChat.avatarThumbnailURL ? nil : notificationPlaceholderImage;
    
    NSString *title = mostRecentChat.title;
    NSString *detail;
    switch (mostRecentChat.lastMessageType) {
        case PSHChatDAOLastMessageTypeText:
            detail = mostRecentChat.lastMessageBody;
            break;
        case PSHChatDAOLastMessageTypePicture:
            detail = @"Picture";
            break;
        case PSHChatDAOLastMessageTypeVideo:
            detail = @"Video";
            break;
        case PSHChatDAOLastMessageTypeLocation:
            detail = @"Location";
            break;
        case PSHChatDAOLastMessageTypeContact:
            detail = @"Contact";
            break;
        case PSHChatDAOLastMessageTypeCustom:
            detail = @"";
            break;
        default:
            detail = @"";
    }
    
    CMNavBarNotificationView *notificationView =
        [CMNavBarNotificationView notifyWithText:title
                                          detail:detail
                                           image:notificationViewThumbnail
                                     andDuration:2.0];
    
    CGRect frame1 = notificationView.textLabel.frame;
    frame1.origin.y += 5;
    notificationView.textLabel.frame = frame1;
    CGRect frame2 = notificationView.detailTextLabel.frame;
    frame2.origin.y += 3;
    notificationView.detailTextLabel.frame = frame2;
    
    notificationView.textLabel.font = [UIFont boldSystemFontOfSize:12];
    notificationView.detailTextLabel.font = [UIFont systemFontOfSize:12];
    
    if (!notificationViewThumbnail) {
        [notificationView.imageView setImageWithURL:mostRecentChat.avatarThumbnailURL];
    }
}

- (void)onShouldOpenCampaignViewBusEvent:(NSNotification *)notification
{
    PSHShouldOpenCampaignViewBusEvent* event = notification.object;
    [self showCampaign:event.campaign];
}

#pragma mark - Get nav controller for modal vc

- (UINavigationController *)instantiateModalNavigationControllerWithRootViewController:(UIViewController *)newViewController
{
    UIBarButtonItem *rightButton =
        [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                                                      target:self
                                                      action:@selector(onDoneWebBroserButtonClick:)];
    newViewController.navigationItem.rightBarButtonItem = rightButton;
    return [[UINavigationController alloc] initWithRootViewController:newViewController];
}

#pragma mark - Campaigns

- (void)showCampaign:(PSHCampaignDAO *)campaign
{
    if (!_appIsInForeground) {
        _pendingCampaign = campaign;
        
        NSString *alertBody =
            (!campaign.title || [campaign.title isEqualToString:@""]) ? campaign.text : [NSString stringWithFormat:@"%@%@%@",
                                                                                                         campaign.title,
                                                                                                         @" / ",
                                                                                                         campaign.text];
        
        UILocalNotification *localNotification = [[UILocalNotification alloc] init];
        localNotification.fireDate = [NSDate dateWithTimeIntervalSinceNow:1];
        localNotification.alertBody = alertBody;
        localNotification.timeZone = [NSTimeZone defaultTimeZone];
        localNotification.soundName = @"notification.aiff";
        [[UIApplication sharedApplication] scheduleLocalNotification:localNotification];
        
        return;
    }
    else if (_isShowingCampaign) {
        _pendingCampaign = campaign;
        return;
    }
    _isShowingCampaign = YES;
    
    if (!campaign.URL) {
        return;
    }
    
    PSHCampaignWebViewController *webBrowser =
        [[PSHCampaignWebViewController alloc] initWithCampaign:campaign];
    UINavigationController *navCon =
        [self instantiateModalNavigationControllerWithRootViewController:webBrowser];
    [self.window.rootViewController presentViewController:navCon
                                                 animated:YES
                                               completion:nil];
}

- (void)onDoneWebBroserButtonClick:(id)sender
{
    _isShowingCampaign = NO;
    
    [self.window.rootViewController dismissViewControllerAnimated:YES completion:^{
        if (_pendingCampaign) {
            [self showCampaign:_pendingCampaign];
            _pendingCampaign = nil;
        }
    }];
}

#pragma mark - MWPhotoBrowserDelegate

- (NSUInteger)numberOfPhotosInPhotoBrowser:(MWPhotoBrowser *)photoBrowser
{
    return 1;
}

- (MWPhoto *)photoBrowser:(MWPhotoBrowser *)photoBrowser photoAtIndex:(NSUInteger)index
{
    return self.currentPhoto;
}

#pragma mark - Internal / push notification

- (void)handleRemotePushNotification:(NSDictionary *)userInfo completion:(void(^)())completionHandler
{
    NSLog(@"*** Got a new push notification ***");
    
    __weak __typeof(self) weakSelf = self;
    
    void (^onCustomCompletion)(BOOL, PSHCustomDAO *) = ^(BOOL completion,
                                                         PSHCustomDAO *custom) {
        [weakSelf showAlertWithTitle:@"Custom notification!" subtitle:custom.title];
    };
    void (^onCampaignCompletion)(BOOL, PSHCampaignDAO *) = ^(BOOL completion,
                                                             PSHCampaignDAO *campaign) {
        [weakSelf showAlertWithTitle:@"Campaign notification!" subtitle:campaign.title];
    };
    void (^onOtherCompletion)(NSDictionary *) = ^(NSDictionary *userInfo) {
        [self showAlertWithTitle:@"Other notification!" subtitle:@""];
    };
    void (^onFailure)(NSError *error) = ^(NSError *error){
        [self showAlertWithTitle:@"Something went wrong..." subtitle:@""];
    };
    
    PSHEngine *engine = [PSHEngine sharedInstance];
    [engine handleRemotePushWithUserInfo:userInfo
                        completionCustom:onCustomCompletion
                      completionCampaign:onCampaignCompletion
                         completionOther:onOtherCompletion
                                    fail:onFailure];
    
    if (completionHandler)
    {
        completionHandler();
    }
}

- (void)showAlertWithTitle:(NSString *)title subtitle:(NSString *)subtitle
{
    [[[UIAlertView alloc] initWithTitle:title
                                message:subtitle
                               delegate:nil
                      cancelButtonTitle:@"OK!"
                      otherButtonTitles:nil] show];
}

- (void)registerForPushNotifications
{
    UIApplication *application = [UIApplication sharedApplication];
    if ([application respondsToSelector:@selector(registerUserNotificationSettings:)]) {
        //iOS8
        UIUserNotificationType myTypes = (UIUserNotificationTypeBadge |
                                          UIUserNotificationTypeAlert |
                                          UIUserNotificationTypeSound);
        UIUserNotificationSettings *settings =
            [UIUserNotificationSettings settingsForTypes:myTypes
                                              categories:nil];
        [application registerUserNotificationSettings:settings];
        [application registerForRemoteNotifications];
    } else {
        //iOS6, iOS7
        UIRemoteNotificationType myTypes =  UIRemoteNotificationTypeBadge |
                                            UIRemoteNotificationTypeAlert |
                                            UIRemoteNotificationTypeSound;
        [application registerForRemoteNotificationTypes:myTypes];
    }
}

#pragma mark - Internal / app badge

- (void)updateBadges
{
    NSInteger unreadThreads = [PSHChatEngine sharedInstance].numberOfChatsWithUnreadMessages;
    UITabBarItem* chatButtonAtTabBar = [_chatTabBarController.tabBar.items objectAtIndex:2];
    if (unreadThreads > 0) {
        chatButtonAtTabBar.badgeValue = [NSString stringWithFormat:@"%lu", (long)unreadThreads];
    }
    else {
        chatButtonAtTabBar.badgeValue = nil;
    }
    [UIApplication sharedApplication].applicationIconBadgeNumber = unreadThreads;
}

#pragma mark - PSHChatViewDelegate

#define MAKE_ACTIVITY(text,callback) [[PSHChatViewControllerShareActivity alloc] initWithText:text andCallback:callback];

- (void)PSHChatViewDidLoad:(PSHChatViewController *)chatView
{
    PSHChatViewControllerShareActivity *a1 = MAKE_ACTIVITY(@"Take Picture", ^(PSHChatViewController *chatView) {
        [chatView performTakeAndSendPicture];
    });
    PSHChatViewControllerShareActivity *a2 = MAKE_ACTIVITY(@"Take Video", ^(PSHChatViewController *chatView) {
        [chatView performTakeAndSendVideo];
    });
    PSHChatViewControllerShareActivity *a3 = MAKE_ACTIVITY(@"Camera Roll", ^(PSHChatViewController *chatView) {
        [chatView performPickMediaFromCameraRollAndSend];
    });
    PSHChatViewControllerShareActivity *a4 = MAKE_ACTIVITY(@"Send Location", ^(PSHChatViewController *chatView) {
        [chatView performSendLocation];
    });
    PSHChatViewControllerShareActivity *a5 = MAKE_ACTIVITY(@"Send Contact", ^(PSHChatViewController *chatView) {
        [chatView performSendContact];
    });
    
    /*
     PSHTransactionPayload* dummy = [[PSHTransactionPayload alloc] init];
     dummy.exampleBool = NO;
     dummy.exampleFloat = 216.0f;
     dummy.exampleUnsignedInteger = 20;
     dummy.exampleInteger = -612;
     dummy.exampleString = @"HEY!";
     dummy.exampleDate = [NSDate date]; // ignored
     dummy.exampleRect = CGRectZero; // ignored
     */
    
    chatView.shareActivities = @[a1, a2, a3, a4, a5];
}

- (void)PSHChatView:(PSHChatViewController *)chatView didTapOnImageWithURL:(NSURL *)imageURL
{
    self.currentPhoto = [MWPhoto photoWithURL:imageURL];
    
    MWPhotoBrowser *browser = [[MWPhotoBrowser alloc] initWithDelegate:self];
    browser.displayActionButton = NO;
    
    [chatView.navigationController pushViewController:browser animated:YES];
}

- (void)PSHChatView:(PSHChatViewController *)chatView didSelectLinkWithURL:(NSURL *)url
{
    [[UIApplication sharedApplication] openURL:url];
}

- (void)PSHChatView:(PSHChatViewController *)chatView didSelectLinkWithPhoneNumber:(NSURL *)url
{
    [[UIApplication sharedApplication] openURL:url];
}

- (void)PSHChatView:(PSHChatViewController *)chatView shouldPushContactViewController:(ABPersonViewController *)newPersonViewController
{
    [chatView.navigationController pushViewController:newPersonViewController animated:YES];
}

- (void)PSHChatView:(PSHChatViewController *)chatView shouldPushLocationViewController:(UIViewController *)viewController
{
    [chatView.navigationController pushViewController:viewController animated:YES];
}

- (void)PSHChatView:(PSHChatViewController *)chatView shouldPresentPeoplePickerNavigationController:(ABPeoplePickerNavigationController *)newPeoplerPickerNavigationController
{
    [self.window.rootViewController presentViewController:newPeoplerPickerNavigationController
                                                 animated:YES
                                               completion:nil];
}

- (void)PSHChatView:(PSHChatViewController *)chatView shouldPresentUIImagePickerController:(UIImagePickerController *)newImagePickerController
{
    [self.window.rootViewController presentViewController:newImagePickerController
                                                 animated:YES
                                               completion:nil];
}

- (void)PSHChatView:(PSHChatViewController *)chatView presentMoviePlayerViewControllerAnimated:(MPMoviePlayerViewController *)newPlayerViewController
{
    [self.window.rootViewController presentMoviePlayerViewControllerAnimated:newPlayerViewController];
}

- (void)PSHChatView:(PSHChatViewController *)chatView savedContactViewController:(ABPersonViewController *)newPersonViewController
{
    //
}


@end
